@@ -18,16 +18,17 @@ class BaseHashGenerator(object):
 class ClientIdGenerator(BaseHashGenerator):
     def hash(self):
         """
-        Generate a client_id without colon char as in http://tools.ietf.org/html/rfc2617#section-2
-        for Basic Authentication scheme
+        Generate a client_id for Basic Authentication scheme without colon char
+        as in http://tools.ietf.org/html/rfc2617#section-2
         """
         return oauthlib_generate_client_id(length=40, chars=UNICODE_ASCII_CHARACTER_SET)
 
 
 class ClientSecretGenerator(BaseHashGenerator):
     def hash(self):
-        return oauthlib_generate_client_id(length=oauth2_settings.CLIENT_SECRET_GENERATOR_LENGTH,
-                                           chars=UNICODE_ASCII_CHARACTER_SET)
+        length = oauth2_settings.CLIENT_SECRET_GENERATOR_LENGTH
+        chars = UNICODE_ASCII_CHARACTER_SET
+        return oauthlib_generate_client_id(length=length, chars=chars)
 
 
 def generate_client_id():
