@@ -1,18 +1,10 @@
 from django.contrib.auth import authenticate
 from django.utils.cache import patch_vary_headers
 
-# bastb Django 1.10 has updated Middleware. This code imports the Mixin required to get old-style
-# middleware working again
-# More?
-#  https://docs.djangoproject.com/en/1.10/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware
-try:
-    from django.utils.deprecation import MiddlewareMixin
-    middleware_parent_class = MiddlewareMixin
-except ImportError:
-    middleware_parent_class = object
+from .compat import MiddlewareMixin
 
 
-class OAuth2TokenMiddleware(middleware_parent_class):
+class OAuth2TokenMiddleware(MiddlewareMixin):
     """
     Middleware for OAuth2 user authentication
 
@@ -28,17 +20,17 @@ class OAuth2TokenMiddleware(middleware_parent_class):
     also request._cached_user field makes AuthenticationMiddleware use that instead of the one from
     the session.
 
-    It also adds 'Authorization' to the 'Vary' header. So that django's cache middleware or a
-    reverse proxy can create proper cache keys
+    It also adds "Authorization" to the "Vary" header, so that django's cache middleware or a
+    reverse proxy can create proper cache keys.
     """
     def process_request(self, request):
         # do something only if request contains a Bearer token
-        if request.META.get('HTTP_AUTHORIZATION', '').startswith('Bearer'):
-            if not hasattr(request, 'user') or request.user.is_anonymous():
+        if request.META.get("HTTP_AUTHORIZATION", "").startswith("Bearer"):
+            if not hasattr(request, "user") or request.user.is_anonymous:
                 user = authenticate(request=request)
                 if user:
                     request.user = request._cached_user = user
 
     def process_response(self, request, response):
-        patch_vary_headers(response, ('Authorization',))
+        patch_vary_headers(response, ("Authorization",))
         return response
