@@ -4,17 +4,17 @@ from datetime import timedelta
 
 from django.apps import apps
 from django.conf import settings
-from django.core.urlresolvers import reverse
-from django.db import models, transaction
-from django.utils import timezone
-
-from django.utils.translation import ugettext_lazy as _
-from django.utils.encoding import python_2_unicode_compatible
 from django.core.exceptions import ImproperlyConfigured
+from django.db import models, transaction
+from django.urls import reverse
+from django.utils import timezone
+from django.utils.encoding import python_2_unicode_compatible
+from django.utils.translation import ugettext_lazy as _
 
-from .settings import oauth2_settings
 from .compat import parse_qsl, urlparse
-from .generators import generate_client_secret, generate_client_id
+from .generators import generate_client_id, generate_client_secret
+from .scopes import get_scopes_backend
+from .settings import oauth2_settings
 from .validators import validate_uris
 
 
@@ -39,40 +39,51 @@ class AbstractApplication(models.Model):
                             the registration process as described in :rfc:`2.2`
     * :attr:`name` Friendly name for the Application
     """
-    CLIENT_CONFIDENTIAL = 'confidential'
-    CLIENT_PUBLIC = 'public'
+    CLIENT_CONFIDENTIAL = "confidential"
+    CLIENT_PUBLIC = "public"
     CLIENT_TYPES = (
-        (CLIENT_CONFIDENTIAL, _('Confidential')),
-        (CLIENT_PUBLIC, _('Public')),
+        (CLIENT_CONFIDENTIAL, _("Confidential")),
+        (CLIENT_PUBLIC, _("Public")),
     )
 
-    GRANT_AUTHORIZATION_CODE = 'authorization-code'
-    GRANT_IMPLICIT = 'implicit'
-    GRANT_PASSWORD = 'password'
-    GRANT_CLIENT_CREDENTIALS = 'client-credentials'
+    GRANT_AUTHORIZATION_CODE = "authorization-code"
+    GRANT_IMPLICIT = "implicit"
+    GRANT_PASSWORD = "password"
+    GRANT_CLIENT_CREDENTIALS = "client-credentials"
     GRANT_TYPES = (
-        (GRANT_AUTHORIZATION_CODE, _('Authorization code')),
-        (GRANT_IMPLICIT, _('Implicit')),
-        (GRANT_PASSWORD, _('Resource owner password-based')),
-        (GRANT_CLIENT_CREDENTIALS, _('Client credentials')),
+        (GRANT_AUTHORIZATION_CODE, _("Authorization code")),
+        (GRANT_IMPLICIT, _("Implicit")),
+        (GRANT_PASSWORD, _("Resource owner password-based")),
+        (GRANT_CLIENT_CREDENTIALS, _("Client credentials")),
     )
 
-    client_id = models.CharField(max_length=100, unique=True,
-                                 default=generate_client_id, db_index=True)
-    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="%(app_label)s_%(class)s",
-                             null=True, blank=True)
+    id = models.BigAutoField(primary_key=True)
+    client_id = models.CharField(
+        max_length=100, unique=True, default=generate_client_id, db_index=True
+    )
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL,
+        related_name="%(app_label)s_%(class)s",
+        null=True, blank=True, on_delete=models.CASCADE
+    )
 
     help_text = _("Allowed URIs list, space separated")
-    redirect_uris = models.TextField(help_text=help_text,
-                                     validators=[validate_uris], blank=True)
+    redirect_uris = models.TextField(
+        blank=True, help_text=help_text, validators=[validate_uris]
+    )
     client_type = models.CharField(max_length=32, choices=CLIENT_TYPES)
-    authorization_grant_type = models.CharField(max_length=32,
-                                                choices=GRANT_TYPES)
-    client_secret = models.CharField(max_length=255, blank=True,
-                                     default=generate_client_secret, db_index=True)
+    authorization_grant_type = models.CharField(
+        max_length=32, choices=GRANT_TYPES
+    )
+    client_secret = models.CharField(
+        max_length=255, blank=True, default=generate_client_secret, db_index=True
+    )
     name = models.CharField(max_length=255, blank=True)
     skip_authorization = models.BooleanField(default=False)
 
+    created = models.DateTimeField(auto_now_add=True)
+    updated = models.DateTimeField(auto_now=True)
+
     class Meta:
         abstract = True
 
@@ -85,9 +96,11 @@ class AbstractApplication(models.Model):
         if self.redirect_uris:
             return self.redirect_uris.split().pop(0)
 
-        assert False, "If you are using implicit, authorization_code" \
-                      "or all-in-one grant_type, you must define " \
-                      "redirect_uris field in your Application model"
+        assert False, (
+            "If you are using implicit, authorization_code"
+            "or all-in-one grant_type, you must define "
+            "redirect_uris field in your Application model"
+        )
 
     def redirect_uri_allowed(self, uri):
         """
@@ -117,23 +130,34 @@ class AbstractApplication(models.Model):
             and self.authorization_grant_type \
             in (AbstractApplication.GRANT_AUTHORIZATION_CODE,
                 AbstractApplication.GRANT_IMPLICIT):
-            error = _('Redirect_uris could not be empty with {0} grant_type')
-            raise ValidationError(error.format(self.authorization_grant_type))
+            error = _("Redirect_uris could not be empty with {grant_type} grant_type")
+            raise ValidationError(error.format(grant_type=self.authorization_grant_type))
 
     def get_absolute_url(self):
-        return reverse('oauth2_provider:detail', args=[str(self.id)])
+        return reverse("oauth2_provider:detail", args=[str(self.id)])
 
     def __str__(self):
         return self.name or self.client_id
 
+    def allows_grant_type(self, *grant_types):
+        return self.authorization_grant_type in grant_types
+
+    def is_usable(self, request):
+        """
+        Determines whether the application can be used.
+
+        :param request: The HTTP request being processed.
+        """
+        return True
+
 
 class Application(AbstractApplication):
     class Meta(AbstractApplication.Meta):
-        swappable = 'OAUTH2_PROVIDER_APPLICATION_MODEL'
+        swappable = "OAUTH2_PROVIDER_APPLICATION_MODEL"
 
 
 @python_2_unicode_compatible
-class Grant(models.Model):
+class AbstractGrant(models.Model):
     """
     A Grant instance represents a token with a short lifetime that can
     be swapped for an access token, as described in :rfc:`4.1.2`
@@ -148,13 +172,22 @@ class Grant(models.Model):
     * :attr:`redirect_uri` Self explained
     * :attr:`scope` Required scopes, optional
     """
-    user = models.ForeignKey(settings.AUTH_USER_MODEL)
+    id = models.BigAutoField(primary_key=True)
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
+        related_name="%(app_label)s_%(class)s"
+    )
     code = models.CharField(max_length=255, unique=True)  # code comes from oauthlib
-    application = models.ForeignKey(oauth2_settings.APPLICATION_MODEL)
+    application = models.ForeignKey(
+        oauth2_settings.APPLICATION_MODEL, on_delete=models.CASCADE
+    )
     expires = models.DateTimeField()
     redirect_uri = models.CharField(max_length=255)
     scope = models.TextField(blank=True)
 
+    created = models.DateTimeField(auto_now_add=True)
+    updated = models.DateTimeField(auto_now=True)
+
     def is_expired(self):
         """
         Check token expiration with timezone awareness
@@ -170,9 +203,17 @@ class Grant(models.Model):
     def __str__(self):
         return self.code
 
+    class Meta:
+        abstract = True
+
+
+class Grant(AbstractGrant):
+    class Meta(AbstractGrant.Meta):
+        swappable = "OAUTH2_PROVIDER_GRANT_MODEL"
+
 
 @python_2_unicode_compatible
-class AccessToken(models.Model):
+class AbstractAccessToken(models.Model):
     """
     An AccessToken instance represents the actual access token to
     access user's resources, as in :rfc:`5`.
@@ -185,12 +226,21 @@ class AccessToken(models.Model):
     * :attr:`expires` Date and time of token expiration, in DateTime format
     * :attr:`scope` Allowed scopes
     """
-    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
-    token = models.CharField(max_length=255, unique=True)
-    application = models.ForeignKey(oauth2_settings.APPLICATION_MODEL)
+    id = models.BigAutoField(primary_key=True)
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, blank=True, null=True,
+        related_name="%(app_label)s_%(class)s"
+    )
+    token = models.CharField(max_length=255, unique=True, )
+    application = models.ForeignKey(
+        oauth2_settings.APPLICATION_MODEL, on_delete=models.CASCADE, blank=True, null=True,
+    )
     expires = models.DateTimeField()
     scope = models.TextField(blank=True)
 
+    created = models.DateTimeField(auto_now_add=True)
+    updated = models.DateTimeField(auto_now=True)
+
     def is_valid(self, scopes=None):
         """
         Checks if the access token is valid.
@@ -234,14 +284,24 @@ class AccessToken(models.Model):
         """
         Returns a dictionary of allowed scope names (as keys) with their descriptions (as values)
         """
-        return {name: desc for name, desc in oauth2_settings.SCOPES.items() if name in self.scope.split()}
+        all_scopes = get_scopes_backend().get_all_scopes()
+        token_scopes = self.scope.split()
+        return {name: desc for name, desc in all_scopes.items() if name in token_scopes}
 
     def __str__(self):
         return self.token
 
+    class Meta:
+        abstract = True
+
+
+class AccessToken(AbstractAccessToken):
+    class Meta(AbstractAccessToken.Meta):
+        swappable = "OAUTH2_PROVIDER_ACCESS_TOKEN_MODEL"
+
 
 @python_2_unicode_compatible
-class RefreshToken(models.Model):
+class AbstractRefreshToken(models.Model):
     """
     A RefreshToken instance represents a token that can be swapped for a new
     access token when it expires.
@@ -254,41 +314,68 @@ class RefreshToken(models.Model):
     * :attr:`access_token` AccessToken instance this refresh token is
                            bounded to
     """
-    user = models.ForeignKey(settings.AUTH_USER_MODEL)
+    id = models.BigAutoField(primary_key=True)
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL, on_delete=models.CASCADE,
+        related_name="%(app_label)s_%(class)s"
+    )
     token = models.CharField(max_length=255, unique=True)
-    application = models.ForeignKey(oauth2_settings.APPLICATION_MODEL)
-    access_token = models.OneToOneField(AccessToken,
-                                        related_name='refresh_token')
+    application = models.ForeignKey(
+        oauth2_settings.APPLICATION_MODEL, on_delete=models.CASCADE)
+    access_token = models.OneToOneField(
+        oauth2_settings.ACCESS_TOKEN_MODEL, on_delete=models.CASCADE,
+        related_name="refresh_token"
+    )
+
+    created = models.DateTimeField(auto_now_add=True)
+    updated = models.DateTimeField(auto_now=True)
 
     def revoke(self):
         """
         Delete this refresh token along with related access token
         """
-        AccessToken.objects.get(id=self.access_token.id).revoke()
+        access_token_model = get_access_token_model()
+        access_token_model.objects.get(id=self.access_token.id).revoke()
         self.delete()
 
     def __str__(self):
         return self.token
 
+    class Meta:
+        abstract = True
+
+
+class RefreshToken(AbstractRefreshToken):
+    class Meta(AbstractRefreshToken.Meta):
+        swappable = "OAUTH2_PROVIDER_REFRESH_TOKEN_MODEL"
+
 
 def get_application_model():
     """ Return the Application model that is active in this project. """
-    try:
-        app_label, model_name = oauth2_settings.APPLICATION_MODEL.split('.')
-    except ValueError:
-        e = "APPLICATION_MODEL must be of the form 'app_label.model_name'"
-        raise ImproperlyConfigured(e)
-    app_model = apps.get_model(app_label, model_name)
-    if app_model is None:
-        e = "APPLICATION_MODEL refers to model {0} that has not been installed"
-        raise ImproperlyConfigured(e.format(oauth2_settings.APPLICATION_MODEL))
-    return app_model
+    return apps.get_model(oauth2_settings.APPLICATION_MODEL)
+
+
+def get_grant_model():
+    """ Return the Grant model that is active in this project. """
+    return apps.get_model(oauth2_settings.GRANT_MODEL)
+
+
+def get_access_token_model():
+    """ Return the AccessToken model that is active in this project. """
+    return apps.get_model(oauth2_settings.ACCESS_TOKEN_MODEL)
+
+
+def get_refresh_token_model():
+    """ Return the RefreshToken model that is active in this project. """
+    return apps.get_model(oauth2_settings.REFRESH_TOKEN_MODEL)
 
 
 def clear_expired():
     now = timezone.now()
     refresh_expire_at = None
-
+    access_token_model = get_access_token_model()
+    refresh_token_model = get_refresh_token_model()
+    grant_model = get_grant_model()
     REFRESH_TOKEN_EXPIRE_SECONDS = oauth2_settings.REFRESH_TOKEN_EXPIRE_SECONDS
     if REFRESH_TOKEN_EXPIRE_SECONDS:
         if not isinstance(REFRESH_TOKEN_EXPIRE_SECONDS, timedelta):
@@ -301,6 +388,6 @@ def clear_expired():
 
     with transaction.atomic():
         if refresh_expire_at:
-            RefreshToken.objects.filter(access_token__expires__lt=refresh_expire_at).delete()
-        AccessToken.objects.filter(refresh_token__isnull=True, expires__lt=now).delete()
-        Grant.objects.filter(expires__lt=now).delete()
+            refresh_token_model.objects.filter(access_token__expires__lt=refresh_expire_at).delete()
+            access_token_model.objects.filter(refresh_token__isnull=True, expires__lt=now).delete()
+        grant_model.objects.filter(expires__lt=now).delete()
