@@ -1,44 +1,63 @@
 from __future__ import unicode_literals
 
-import six
 import base64
 import binascii
 import logging
-from datetime import timedelta
+from datetime import datetime, timedelta
 
-from django.utils import timezone
+import requests
 from django.conf import settings
-from django.contrib.auth import authenticate
+from django.contrib.auth import authenticate, get_user_model
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import transaction
+from django.utils import timezone
+from django.utils.timezone import make_aware
 from oauthlib.oauth2 import RequestValidator
 
 from .compat import unquote_plus
 from .exceptions import FatalClientError
-from .models import Grant, AccessToken, RefreshToken, get_application_model, AbstractApplication
+from .models import (
+    AbstractApplication,
+    get_access_token_model,
+    get_application_model,
+    get_grant_model,
+    get_refresh_token_model,
+)
+from .scopes import get_scopes_backend
 from .settings import oauth2_settings
 
-log = logging.getLogger('oauth2_provider')
+
+log = logging.getLogger("oauth2_provider")
 
 GRANT_TYPE_MAPPING = {
-    'authorization_code': (AbstractApplication.GRANT_AUTHORIZATION_CODE,),
-    'password': (AbstractApplication.GRANT_PASSWORD,),
-    'client_credentials': (AbstractApplication.GRANT_CLIENT_CREDENTIALS,),
-    'refresh_token': (AbstractApplication.GRANT_AUTHORIZATION_CODE, AbstractApplication.GRANT_PASSWORD,
-                      AbstractApplication.GRANT_CLIENT_CREDENTIALS)
+    "authorization_code": (AbstractApplication.GRANT_AUTHORIZATION_CODE, ),
+    "password": (AbstractApplication.GRANT_PASSWORD, ),
+    "client_credentials": (AbstractApplication.GRANT_CLIENT_CREDENTIALS, ),
+    "refresh_token": (
+        AbstractApplication.GRANT_AUTHORIZATION_CODE,
+        AbstractApplication.GRANT_PASSWORD,
+        AbstractApplication.GRANT_CLIENT_CREDENTIALS,
+    )
 }
 
+Application = get_application_model()
+AccessToken = get_access_token_model()
+Grant = get_grant_model()
+RefreshToken = get_refresh_token_model()
+UserModel = get_user_model()
+
 
 class OAuth2Validator(RequestValidator):
     def _extract_basic_auth(self, request):
         """
-        Return authentication string if request contains basic auth credentials, else return None
+        Return authentication string if request contains basic auth credentials,
+        otherwise return None
         """
-        auth = request.headers.get('HTTP_AUTHORIZATION', None)
+        auth = request.headers.get("HTTP_AUTHORIZATION", None)
         if not auth:
             return None
 
-        splitted = auth.split(' ', 1)
+        splitted = auth.split(" ", 1)
         if len(splitted) != 2:
             return None
         auth_type, auth_string = splitted
@@ -60,30 +79,26 @@ class OAuth2Validator(RequestValidator):
             return False
 
         try:
-            encoding = request.encoding or settings.DEFAULT_CHARSET or 'utf-8'
+            encoding = request.encoding or settings.DEFAULT_CHARSET or "utf-8"
         except AttributeError:
-            encoding = 'utf-8'
-
-        # Encode auth_string to bytes. This is needed for python3.2 compatibility
-        # because b64decode function only supports bytes type in input.
-        if isinstance(auth_string, six.string_types):
-            auth_string = auth_string.encode(encoding)
+            encoding = "utf-8"
 
         try:
             b64_decoded = base64.b64decode(auth_string)
         except (TypeError, binascii.Error):
-            log.debug("Failed basic auth: %s can't be decoded as base64", auth_string)
+            log.debug("Failed basic auth: %r can't be decoded as base64", auth_string)
             return False
 
         try:
             auth_string_decoded = b64_decoded.decode(encoding)
         except UnicodeDecodeError:
-            log.debug("Failed basic auth: %s can't be decoded as unicode by %s",
-                      auth_string,
-                      encoding)
+            log.debug(
+                "Failed basic auth: %r can't be decoded as unicode by %r",
+                auth_string, encoding
+            )
             return False
 
-        client_id, client_secret = map(unquote_plus, auth_string_decoded.split(':', 1))
+        client_id, client_secret = map(unquote_plus, auth_string_decoded.split(":", 1))
 
         if self._load_application(client_id, request) is None:
             log.debug("Failed basic auth: Application %s does not exist" % client_id)
@@ -99,11 +114,12 @@ class OAuth2Validator(RequestValidator):
 
     def _authenticate_request_body(self, request):
         """
-        Try to authenticate the client using client_id and client_secret parameters
-        included in body.
+        Try to authenticate the client using client_id and client_secret
+        parameters included in body.
 
-        Remember that this method is NOT RECOMMENDED and SHOULD be limited to clients unable to
-        directly utilize the HTTP Basic authentication scheme. See rfc:`2.3.1` for more details.
+        Remember that this method is NOT RECOMMENDED and SHOULD be limited to
+        clients unable to directly utilize the HTTP Basic authentication scheme.
+        See rfc:`2.3.1` for more details.
         """
         # TODO: check if oauthlib has already unquoted client_id and client_secret
         try:
@@ -123,19 +139,22 @@ class OAuth2Validator(RequestValidator):
 
     def _load_application(self, client_id, request):
         """
-        If request.client was not set, load application instance for given client_id and store it
-        in request.client
+        If request.client was not set, load application instance for given
+        client_id and store it in request.client
         """
 
         # we want to be sure that request has the client attribute!
-        assert hasattr(request, "client"), "'request' instance has no 'client' attribute"
+        assert hasattr(request, "client"), '"request" instance has no "client" attribute'
 
-        Application = get_application_model()
         try:
             request.client = request.client or Application.objects.get(client_id=client_id)
+            # Check that the application can be used (defaults to always True)
+            if not request.client.is_usable(request):
+                log.debug("Failed body authentication: Application %r is disabled" % (client_id))
+                return None
             return request.client
         except Application.DoesNotExist:
-            log.debug("Failed body authentication: Application %s does not exist" % client_id)
+            log.debug("Failed body authentication: Application %r does not exist" % (client_id))
             return None
 
     def client_authentication_required(self, request, *args, **kwargs):
@@ -147,11 +166,11 @@ class OAuth2Validator(RequestValidator):
             * Resource owner password grant
             * Refresh token grant
 
-        If the request contains authorization headers, always authenticate the client no matter
-        the grant type.
+        If the request contains authorization headers, always authenticate the client
+        no matter the grant type.
 
-        If the request does not contain authorization headers, proceed with authentication only if
-        the client is of type `Confidential`.
+        If the request does not contain authorization headers, proceed with authentication
+        only if the client is of type `Confidential`.
 
         If something goes wrong, call oauthlib implementation of the method.
         """
@@ -162,7 +181,7 @@ class OAuth2Validator(RequestValidator):
             if request.client_id and request.client_secret:
                 return True
         except AttributeError:
-            log.debug("Client id or client secret not provided, proceed evaluating if authentication is required...")
+            log.debug("Client ID or client secret not provided...")
             pass
 
         self._load_application(request.client_id, request)
@@ -174,13 +193,14 @@ class OAuth2Validator(RequestValidator):
 
     def authenticate_client(self, request, *args, **kwargs):
         """
-        Check if client exists and it's authenticating itself as in rfc:`3.2.1`
+        Check if client exists and is authenticating itself as in rfc:`3.2.1`
 
         First we try to authenticate with HTTP Basic Auth, and that is the PREFERRED
         authentication method.
-        Whether this fails we support including the client credentials in the request-body, but
-        this method is NOT RECOMMENDED and SHOULD be limited to clients unable to directly utilize
-        the HTTP Basic authentication scheme. See rfc:`2.3.1` for more details
+        Whether this fails we support including the client credentials in the request-body,
+        but this method is NOT RECOMMENDED and SHOULD be limited to clients unable to
+        directly utilize the HTTP Basic authentication scheme.
+        See rfc:`2.3.1` for more details
         """
         authenticated = self._authenticate_basic_auth(request)
 
@@ -192,11 +212,10 @@ class OAuth2Validator(RequestValidator):
     def authenticate_client_id(self, client_id, request, *args, **kwargs):
         """
         If we are here, the client did not authenticate itself as in rfc:`3.2.1` and we can
-        proceed only if the client exists and it's not of type 'Confidential'.
-        Also assign Application instance to request.client.
+        proceed only if the client exists and is not of type "Confidential".
         """
         if self._load_application(client_id, request) is not None:
-            log.debug("Application %s has type %s" % (client_id, request.client.client_type))
+            log.debug("Application %r has type %r" % (client_id, request.client.client_type))
             return request.client.client_type != AbstractApplication.CLIENT_CONFIDENTIAL
         return False
 
@@ -216,14 +235,71 @@ class OAuth2Validator(RequestValidator):
 
     def validate_client_id(self, client_id, request, *args, **kwargs):
         """
-        Ensure an Application exists with given client_id. If it exists, it's assigned to
-        request.client.
+        Ensure an Application exists with given client_id.
+        If it exists, it's assigned to request.client.
         """
         return self._load_application(client_id, request) is not None
 
     def get_default_redirect_uri(self, client_id, request, *args, **kwargs):
         return request.client.default_redirect_uri
 
+    def _get_token_from_authentication_server(self, token, introspection_url, introspection_token):
+        bearer = "Bearer {}".format(introspection_token)
+
+        try:
+            response = requests.post(
+                introspection_url,
+                data={"token": token}, headers={"Authorization": bearer}
+            )
+        except requests.exceptions.RequestException:
+            log.exception("Introspection: Failed POST to %r in token lookup", introspection_url)
+            return None
+
+        try:
+            content = response.json()
+        except ValueError:
+            log.exception("Introspection: Failed to parse response as json")
+            return None
+
+        if "active" in content and content["active"] is True:
+            if "username" in content:
+                user, _created = UserModel.objects.get_or_create(
+                    **{UserModel.USERNAME_FIELD: content["username"]}
+                )
+            else:
+                user = None
+
+            max_caching_time = datetime.now() + timedelta(
+                seconds=oauth2_settings.RESOURCE_SERVER_TOKEN_CACHING_SECONDS
+            )
+
+            if "exp" in content:
+                expires = datetime.utcfromtimestamp(content["exp"])
+                if expires > max_caching_time:
+                    expires = max_caching_time
+            else:
+                expires = max_caching_time
+
+            scope = content.get("scope", "")
+            expires = make_aware(expires)
+
+            try:
+                access_token = AccessToken.objects.select_related("application", "user").get(token=token)
+            except AccessToken.DoesNotExist:
+                access_token = AccessToken.objects.create(
+                    token=token,
+                    user=user,
+                    application=None,
+                    scope=scope,
+                    expires=expires
+                )
+            else:
+                access_token.expires = expires
+                access_token.scope = scope
+                access_token.save()
+
+            return access_token
+
     def validate_bearer_token(self, token, scopes, request):
         """
         When users try to access resources, check that provided token is valid
@@ -231,10 +307,20 @@ class OAuth2Validator(RequestValidator):
         if not token:
             return False
 
+        introspection_url = oauth2_settings.RESOURCE_SERVER_INTROSPECTION_URL
+        introspection_token = oauth2_settings.RESOURCE_SERVER_AUTH_TOKEN
+
         try:
-            access_token = AccessToken.objects.select_related("application", "user").get(
-                token=token)
-            if access_token.is_valid(scopes):
+            access_token = AccessToken.objects.select_related("application", "user").get(token=token)
+            # if there is a token but invalid then look up the token
+            if introspection_url and introspection_token:
+                if not access_token.is_valid(scopes):
+                    access_token = self._get_token_from_authentication_server(
+                        token,
+                        introspection_url,
+                        introspection_token
+                    )
+            if access_token and access_token.is_valid(scopes):
                 request.client = access_token.application
                 request.user = access_token.user
                 request.scopes = scopes
@@ -244,13 +330,28 @@ class OAuth2Validator(RequestValidator):
                 return True
             return False
         except AccessToken.DoesNotExist:
+            # there is no initial token, look up the token
+            if introspection_url and introspection_token:
+                access_token = self._get_token_from_authentication_server(
+                    token,
+                    introspection_url,
+                    introspection_token
+                )
+                if access_token and access_token.is_valid(scopes):
+                    request.client = access_token.application
+                    request.user = access_token.user
+                    request.scopes = scopes
+
+                    # this is needed by django rest framework
+                    request.access_token = access_token
+                    return True
             return False
 
     def validate_code(self, client_id, code, client, request, *args, **kwargs):
         try:
             grant = Grant.objects.get(code=code, application=client)
             if not grant.is_expired():
-                request.scopes = grant.scope.split(' ')
+                request.scopes = grant.scope.split(" ")
                 request.user = grant.user
                 return True
             return False
@@ -263,17 +364,17 @@ class OAuth2Validator(RequestValidator):
         Validate both grant_type is a valid string and grant_type is allowed for current workflow
         """
         assert(grant_type in GRANT_TYPE_MAPPING)  # mapping misconfiguration
-        return request.client.authorization_grant_type in GRANT_TYPE_MAPPING[grant_type]
+        return request.client.allows_grant_type(*GRANT_TYPE_MAPPING[grant_type])
 
     def validate_response_type(self, client_id, response_type, client, request, *args, **kwargs):
         """
         We currently do not support the Authorization Endpoint Response Types registry as in
-        rfc:`8.4`, so validate the response_type only if it matches 'code' or 'token'
+        rfc:`8.4`, so validate the response_type only if it matches "code" or "token"
         """
-        if response_type == 'code':
-            return client.authorization_grant_type == AbstractApplication.GRANT_AUTHORIZATION_CODE
-        elif response_type == 'token':
-            return client.authorization_grant_type == AbstractApplication.GRANT_IMPLICIT
+        if response_type == "code":
+            return client.allows_grant_type(AbstractApplication.GRANT_AUTHORIZATION_CODE)
+        elif response_type == "token":
+            return client.allows_grant_type(AbstractApplication.GRANT_IMPLICIT)
         else:
             return False
 
@@ -281,10 +382,12 @@ class OAuth2Validator(RequestValidator):
         """
         Ensure required scopes are permitted (as specified in the settings file)
         """
-        return set(scopes).issubset(set(oauth2_settings._SCOPES))
+        available_scopes = get_scopes_backend().get_available_scopes(application=client, request=request)
+        return set(scopes).issubset(set(available_scopes))
 
     def get_default_scopes(self, client_id, request, *args, **kwargs):
-        return oauth2_settings._DEFAULT_SCOPES
+        default_scopes = get_scopes_backend().get_default_scopes(application=request.client, request=request)
+        return default_scopes
 
     def validate_redirect_uri(self, client_id, redirect_uri, request, *args, **kwargs):
         return request.client.redirect_uri_allowed(redirect_uri)
@@ -292,9 +395,9 @@ class OAuth2Validator(RequestValidator):
     def save_authorization_code(self, client_id, code, request, *args, **kwargs):
         expires = timezone.now() + timedelta(
             seconds=oauth2_settings.AUTHORIZATION_CODE_EXPIRE_SECONDS)
-        g = Grant(application=request.client, user=request.user, code=code['code'],
+        g = Grant(application=request.client, user=request.user, code=code["code"],
                   expires=expires, redirect_uri=request.redirect_uri,
-                  scope=' '.join(request.scopes))
+                  scope=" ".join(request.scopes))
         g.save()
 
     def rotate_refresh_token(self, request):
@@ -312,12 +415,12 @@ class OAuth2Validator(RequestValidator):
         @see: https://tools.ietf.org/html/draft-ietf-oauth-v2-31#page-43
         """
 
-        if 'scope' not in token:
+        if "scope" not in token:
             raise FatalClientError("Failed to renew access token: missing scope")
 
         expires = timezone.now() + timedelta(seconds=oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS)
 
-        if request.grant_type == 'client_credentials':
+        if request.grant_type == "client_credentials":
             request.user = None
 
         # This comes from OAuthLib:
@@ -325,12 +428,12 @@ class OAuth2Validator(RequestValidator):
         # Its value is either a new random code; or if we are reusing
         # refresh tokens, then it is the same value that the request passed in
         # (stored in `request.refresh_token`)
-        refresh_token_code = token.get('refresh_token', None)
+        refresh_token_code = token.get("refresh_token", None)
 
         if refresh_token_code:
             # an instance of `RefreshToken` that matches the old refresh code.
             # Set on the request in `validate_refresh_token`
-            refresh_token_instance = getattr(request, 'refresh_token_instance', None)
+            refresh_token_instance = getattr(request, "refresh_token_instance", None)
 
             # If we are to reuse tokens, and we can: do so
             if not self.rotate_refresh_token(request) and \
@@ -341,9 +444,9 @@ class OAuth2Validator(RequestValidator):
                     pk=refresh_token_instance.access_token.pk
                 )
                 access_token.user = request.user
-                access_token.scope = token['scope']
+                access_token.scope = token["scope"]
                 access_token.expires = expires
-                access_token.token = token['access_token']
+                access_token.token = token["access_token"]
                 access_token.application = request.client
                 access_token.save()
 
@@ -356,7 +459,7 @@ class OAuth2Validator(RequestValidator):
                     except (AccessToken.DoesNotExist, RefreshToken.DoesNotExist):
                         pass
                     else:
-                        setattr(request, 'refresh_token_instance', None)
+                        setattr(request, "refresh_token_instance", None)
 
                 access_token = self._create_access_token(expires, request, token)
 
@@ -373,14 +476,14 @@ class OAuth2Validator(RequestValidator):
             self._create_access_token(expires, request, token)
 
         # TODO: check out a more reliable way to communicate expire time to oauthlib
-        token['expires_in'] = oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS
+        token["expires_in"] = oauth2_settings.ACCESS_TOKEN_EXPIRE_SECONDS
 
     def _create_access_token(self, expires, request, token):
         access_token = AccessToken(
             user=request.user,
-            scope=token['scope'],
+            scope=token["scope"],
             expires=expires,
-            token=token['access_token'],
+            token=token["access_token"],
             application=request.client
         )
         access_token.save()
@@ -394,12 +497,12 @@ class OAuth2Validator(RequestValidator):
         :param token_type_hint: access_token or refresh_token.
         :param request: The HTTP Request (oauthlib.common.Request)
         """
-        if token_type_hint not in ['access_token', 'refresh_token']:
+        if token_type_hint not in ["access_token", "refresh_token"]:
             token_type_hint = None
 
         token_types = {
-            'access_token': AccessToken,
-            'refresh_token': RefreshToken,
+            "access_token": AccessToken,
+            "refresh_token": RefreshToken,
         }
 
         token_type = token_types.get(token_type_hint, AccessToken)
