@@ -1,8 +1,7 @@
-from django.core.urlresolvers import reverse_lazy
+from django.contrib.auth.mixins import LoginRequiredMixin
 from django.forms.models import modelform_factory
-from django.views.generic import CreateView, DetailView, DeleteView, ListView, UpdateView
-
-from braces.views import LoginRequiredMixin
+from django.urls import reverse_lazy
+from django.views.generic import CreateView, DeleteView, DetailView, ListView, UpdateView
 
 from ..models import get_application_model
 
@@ -11,7 +10,7 @@ class ApplicationOwnerIsUserMixin(LoginRequiredMixin):
     """
     This mixin is used to provide an Application queryset filtered by the current request.user.
     """
-    fields = '__all__'
+    fields = "__all__"
 
     def get_queryset(self):
         return get_application_model().objects.filter(user=self.request.user)
@@ -29,8 +28,10 @@ class ApplicationRegistration(LoginRequiredMixin, CreateView):
         """
         return modelform_factory(
             get_application_model(),
-            fields=('name', 'client_id', 'client_secret', 'client_type',
-                    'authorization_grant_type', 'redirect_uris')
+            fields=(
+                "name", "client_id", "client_secret", "client_type",
+                "authorization_grant_type", "redirect_uris"
+            )
         )
 
     def form_valid(self, form):
@@ -42,7 +43,7 @@ class ApplicationDetail(ApplicationOwnerIsUserMixin, DetailView):
     """
     Detail view for an application instance owned by the request.user
     """
-    context_object_name = 'application'
+    context_object_name = "application"
     template_name = "oauth2_provider/application_detail.html"
 
 
@@ -50,7 +51,7 @@ class ApplicationList(ApplicationOwnerIsUserMixin, ListView):
     """
     List view for all the applications owned by the request.user
     """
-    context_object_name = 'applications'
+    context_object_name = "applications"
     template_name = "oauth2_provider/application_list.html"
 
 
@@ -58,8 +59,8 @@ class ApplicationDelete(ApplicationOwnerIsUserMixin, DeleteView):
     """
     View used to delete an application owned by the request.user
     """
-    context_object_name = 'application'
-    success_url = reverse_lazy('oauth2_provider:list')
+    context_object_name = "application"
+    success_url = reverse_lazy("oauth2_provider:list")
     template_name = "oauth2_provider/application_confirm_delete.html"
 
 
@@ -67,5 +68,17 @@ class ApplicationUpdate(ApplicationOwnerIsUserMixin, UpdateView):
     """
     View used to update an application owned by the request.user
     """
-    context_object_name = 'application'
+    context_object_name = "application"
     template_name = "oauth2_provider/application_form.html"
+
+    def get_form_class(self):
+        """
+        Returns the form class for the application model
+        """
+        return modelform_factory(
+            get_application_model(),
+            fields=(
+                "name", "client_id", "client_secret", "client_type",
+                "authorization_grant_type", "redirect_uris"
+            )
+        )
