@@ -1,7 +1,7 @@
 from django.views.generic import View
 
+from .mixins import ProtectedResourceMixin, ReadWriteScopedResourceMixin, ScopedResourceMixin
 from ..settings import oauth2_settings
-from .mixins import ProtectedResourceMixin, ScopedResourceMixin, ReadWriteScopedResourceMixin
 
 
 class ProtectedResourceView(ProtectedResourceMixin, View):
