@@ -6,12 +6,13 @@ from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpResponseForbidden
 
 from ..exceptions import FatalClientError
+from ..scopes import get_scopes_backend
 from ..settings import oauth2_settings
 
 
 log = logging.getLogger("oauth2_provider")
 
-SAFE_HTTP_METHODS = ['GET', 'HEAD', 'OPTIONS']
+SAFE_HTTP_METHODS = ["GET", "HEAD", "OPTIONS"]
 
 
 class OAuthLibMixin(object):
@@ -82,7 +83,7 @@ class OAuthLibMixin(object):
         """
         Cache and return `OAuthlibCore` instance so it will be created only on first request
         """
-        if not hasattr(cls, '_oauthlib_core'):
+        if not hasattr(cls, "_oauthlib_core"):
             server = cls.get_server()
             core_class = cls.get_oauthlib_backend_class()
             cls._oauthlib_core = core_class(server)
@@ -144,7 +145,8 @@ class OAuthLibMixin(object):
 
     def get_scopes(self):
         """
-        This should return the list of scopes required to access the resources. By default it returns an empty list
+        This should return the list of scopes required to access the resources.
+        By default it returns an empty list.
         """
         return []
 
@@ -157,11 +159,11 @@ class OAuthLibMixin(object):
         oauthlib_error = error.oauthlib_error
 
         redirect_uri = oauthlib_error.redirect_uri or ""
-        separator = '&' if '?' in redirect_uri else '?'
+        separator = "&" if "?" in redirect_uri else "?"
 
         error_response = {
-            'error': oauthlib_error,
-            'url': "{0}{1}{2}".format(oauthlib_error.redirect_uri, separator, oauthlib_error.urlencoded)
+            "error": oauthlib_error,
+            "url": redirect_uri + separator + oauthlib_error.urlencoded,
         }
         error_response.update(kwargs)
 
@@ -201,7 +203,7 @@ class ProtectedResourceMixin(OAuthLibMixin):
     """
     def dispatch(self, request, *args, **kwargs):
         # let preflight OPTIONS requests pass
-        if request.method.upper() == 'OPTIONS':
+        if request.method.upper() == "OPTIONS":
             return super(ProtectedResourceMixin, self).dispatch(request, *args, **kwargs)
 
         # check if the request is valid and the protected resource may be accessed
@@ -221,13 +223,13 @@ class ReadWriteScopedResourceMixin(ScopedResourceMixin, OAuthLibMixin):
     read_write_scope = None
 
     def __new__(cls, *args, **kwargs):
-        provided_scopes = oauth2_settings._SCOPES
+        provided_scopes = get_scopes_backend().get_all_scopes()
         read_write_scopes = [oauth2_settings.READ_SCOPE, oauth2_settings.WRITE_SCOPE]
 
         if not set(read_write_scopes).issubset(set(provided_scopes)):
             raise ImproperlyConfigured(
-                "ReadWriteScopedResourceMixin requires following scopes {0}"
-                " to be in OAUTH2_PROVIDER['SCOPES'] list in settings".format(read_write_scopes)
+                "ReadWriteScopedResourceMixin requires following scopes {}"
+                ' to be in OAUTH2_PROVIDER["SCOPES"] list in settings'.format(read_write_scopes)
             )
 
         return super(ReadWriteScopedResourceMixin, cls).__new__(cls, *args, **kwargs)
@@ -242,4 +244,6 @@ class ReadWriteScopedResourceMixin(ScopedResourceMixin, OAuthLibMixin):
 
     def get_scopes(self, *args, **kwargs):
         scopes = super(ReadWriteScopedResourceMixin, self).get_scopes(*args, **kwargs)
-        return scopes + [self.read_write_scope]  # this returns a copy so that self.required_scopes is not modified
+
+        # this returns a copy so that self.required_scopes is not modified
+        return scopes + [self.read_write_scope]
